@@ -12,7 +12,6 @@ from _emerge.EbuildMerge import EbuildMerge
 from _emerge.EbuildBuildDir import EbuildBuildDir
 from _emerge.SpawnProcess import SpawnProcess
 from portage.eapi import eapi_exports_replace_vars
-from portage.exception import PortageException
 from portage.output import colorize
 from portage.util import ensure_dirs
 from portage.util._async.AsyncTaskFuture import AsyncTaskFuture
@@ -393,7 +392,17 @@ class Binpkg(CompositeTask):
 
     def _unpack_metadata_exit(self, unpack_metadata):
         if self._default_exit(unpack_metadata) != os.EX_OK:
-            unpack_metadata.future.result()
+            try:
+                unpack_metadata.future.result()
+            except Exception as e:
+                self._writemsg_level(
+                    colorize(
+                        "BAD",
+                        f"!!! Error Extracting '{self._pkg_path}', {e}\n",
+                    ),
+                    noiselevel=-1,
+                    level=logging.ERROR,
+                )
             self._async_unlock_builddir(returncode=self.returncode)
             return
 
@@ -427,18 +436,15 @@ class Binpkg(CompositeTask):
         if self._default_exit(unpack_contents) != os.EX_OK:
             try:
                 unpack_contents.future.result()
-                err = ""
-            except PortageException as e:
-                err = e
-
-            self._writemsg_level(
-                colorize(
-                    "BAD",
-                    f"!!! Error Extracting '{self._pkg_path}', {err}\n",
-                ),
-                noiselevel=-1,
-                level=logging.ERROR,
-            )
+            except Exception as e:
+                self._writemsg_level(
+                    colorize(
+                        "BAD",
+                        f"!!! Error Extracting '{self._pkg_path}', {e}\n",
+                    ),
+                    noiselevel=-1,
+                    level=logging.ERROR,
+                )
             self._async_unlock_builddir(returncode=self.returncode)
             return
 
